//---------------------------------------------------------
// Backlight_test.cpp
//
// Tests GC Plus, GMaster and Bitsy backlight control.
// 
// The application ramps the baclight voltage between its limits
// in a triangle sequence (up, down, up...).
//
//
// Applied Data Systems
//
// HISTORY
// 15may02 20-11151 ak
//   created based on Vee test (20-11061)
//  - add on/off when PWM level=255
//  - add functions to turn backlight on and off
//  - run ramp slower (25 vs 15 ms/step)
// 10jan03 20-11152 ak
//  - update to use adsmartio_load functions
// 10jul03 720020-11153 ak
//  - add defines for backlights that have opposite
//    polarity for on/off signal; see settings below
//    for ON_OFF_LS520 and ON_OFF_LS520_REVERSED
//---------------------------------------------------------


#include "stdafx.h"
#include "ADSmartIO_load.h"

#define VERSION L"720020-11153"

#define DIRECT   1    // Backlight PWM down -> backlight down
#define INVERSE  0    // Backlight PWM up   -> backlight down

#define RAMP_UP 1
#define RAMP_DOWN 0

#define IOCTL_SET_BACKLIGHT_UP    0x00
#define IOCTL_SET_BACKLIGHT_DOWN  0x01
#define IOCTL_SET_BACKLIGHT_ON    0x02
#define IOCTL_SET_BACKLIGHT_OFF   0x03
#define IOCTL_SET_BACKLIGHT_LEVEL 0x04

#define IOCTL_SET_VEE_UP      0x10
#define IOCTL_SET_VEE_DOWN    0x11
#define IOCTL_SET_VEE_ON      0x12
#define IOCTL_SET_VEE_OFF     0x13
#define IOCTL_SET_VEE_LEVEL   0x14

// Uncomment ONLY ONE of the following defines depending whether
// your backlight has polarity same as or opposite from the 
// Taiyo-Yuden (Xentek) LS520 inverter.
#define ON_OFF_LS520
//#define ON_OFF_LS520_REVERSED

//---------------------------------------------------------
// Set PWM level for backlight (SA1111 PWM0)
//---------------------------------------------------------
void SetSA1111PWM0 (BYTE value)
{
  TCHAR SA11[] = TEXT("SAM1:");
  BOOL bRet;
  ULONG BytesRet;
  BYTE ControlVal, RetVal;
  HANDLE hPort = INVALID_HANDLE_VALUE;

  // Open SA1111 driver
  hPort = CreateFile (SA11,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);
  if(hPort == INVALID_HANDLE_VALUE)
  {
    RETAILMSG(1,(L" Can't open %s. \r\n",SA11));
    return;
  }

  // Set the value
  ControlVal = value;
  bRet = DeviceIoControl( hPort,
              IOCTL_SET_BACKLIGHT_LEVEL,
              &ControlVal,
              sizeof(ControlVal),
              &RetVal,
              sizeof(RetVal),
              &BytesRet,
              NULL);
  if(bRet == FALSE)
  {
    RETAILMSG(1,(L" Error from DeviceIoControl().\r\n"));
  }

  CloseHandle(hPort);
}


//---------------------------------------------------------
// Turn backlight ON using SA-1111
//---------------------------------------------------------
void SetSA1111BacklightOn ()
{
  TCHAR SA11[] = TEXT("SAM1:");
  BOOL bRet;
  ULONG BytesRet;
  BYTE ControlVal, RetVal;
  HANDLE hPort = INVALID_HANDLE_VALUE;

  // Open SA1111 driver
  hPort = CreateFile (SA11,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);
  if(hPort == INVALID_HANDLE_VALUE)
  {
    RETAILMSG(1,(L" Can't open %s. \r\n",SA11));
    return;
  }

  // Set the value
  ControlVal = 0; // (control value not used)
  bRet = DeviceIoControl( hPort,
              IOCTL_SET_BACKLIGHT_ON,
              &ControlVal,
              sizeof(ControlVal),
              &RetVal,
              sizeof(RetVal),
              &BytesRet,
              NULL);
  if(bRet == FALSE)
  {
    RETAILMSG(1,(L" Error from DeviceIoControl().\r\n"));
  }

  CloseHandle(hPort);
}


//---------------------------------------------------------
// Turn backlight OFF using SA-1111
//---------------------------------------------------------
void SetSA1111BacklightOff ()
{
  TCHAR SA11[] = TEXT("SAM1:");
  BOOL bRet;
  ULONG BytesRet;
  BYTE ControlVal, RetVal;
  HANDLE hPort = INVALID_HANDLE_VALUE;

  // Open SA1111 driver
  hPort = CreateFile (SA11,GENERIC_READ|GENERIC_WRITE,0,NULL,OPEN_EXISTING,0,NULL);
  if(hPort == INVALID_HANDLE_VALUE)
  {
    RETAILMSG(1,(L" Can't open %s. \r\n",SA11));
    return;
  }

  // Set the value
  ControlVal = 0; // (control value not used)
  bRet = DeviceIoControl( hPort,
              IOCTL_SET_BACKLIGHT_OFF,
              &ControlVal,
              sizeof(ControlVal),
              &RetVal,
              sizeof(RetVal),
              &BytesRet,
              NULL);
  if(bRet == FALSE)
  {
    RETAILMSG(1,(L" Error from DeviceIoControl().\r\n"));
  }

  CloseHandle(hPort);
}


//---------------------------------------------------------
// Links to the available ADSmartIO library or local
// function to perform the tests.
//---------------------------------------------------------
int SelectLibrary ()
{

  if (int LinkErrors = LoadADSmartIOFunctions())
  {
    RETAILMSG(TRUE,(L"! %d ADSmartIO functions failed to link.\r\n",LinkErrors));
    RETAILMSG(TRUE,(L"Program aborted.\r\n"));
    return 1;
  }

  // If GMaster, link to different driver functions for backlight control
  if (ADSmartIO_DLL_id() == SIO_DLL_SMARTIO)
  {
    SIOBacklightPwm = (SIO_VOID_BYTE)SetSA1111PWM0;
    SIOBacklightOn  = (SIO_VOID_VOID)SetSA1111BacklightOn;
    SIOBacklightOff = (SIO_VOID_VOID)SetSA1111BacklightOff;
  }
	
  return 0;

}

//---------------------------------------------------------
//---------------------------------------------------------
void RunRamp (BYTE  Start,
        BYTE  Stop,
        BYTE  Step,
        DWORD sleep_duration)
{
  int level;

  DEBUGMSG(1,(L" Ramping from %d to %d in steps of %d...\r\n",Start,Stop,Step));

  if (Start < Stop)  // ramp up
    for (level=Start;level<=Stop;level=level+Step)
    {
      DEBUGMSG(1,(L"%4d",level));
      SIOBacklightPwm(level);
      Sleep(sleep_duration);
    }
  else               // ramp down
    for (level=Start;level>=Stop;level=level-Step)
    {
      DEBUGMSG(1,(L"%4d",level));
      SIOBacklightPwm(level);
      Sleep(sleep_duration);
    }

  DEBUGMSG(1,(L"\r\n"));
}

//---------------------------------------------------------
// Ramps the backlight between limits
//   ramp_dir   direction to ramp; use RAMP_UP/DOWN constants
//        where RAMP_UP goes from min to max
//   polarity   use DIRECT/INVERSE constants, where
//        DIRECT means that when the PWM output voltage 
//        is 0V, backlight is at its lowest level
//---------------------------------------------------------
void BacklightRamp (int ramp_dir,
        int polarity)
{
  BYTE Min = 0;
  BYTE Max = 255;
  BYTE Step = 5;
  DWORD sleep_dur = 25;

  if (polarity==DIRECT)
    if (ramp_dir==RAMP_UP)
      RunRamp (Min,Max,Step,sleep_dur);
    else
      RunRamp (Max,Min,Step,sleep_dur);
  else                    // 0V is brightest
  {
    if (ramp_dir==RAMP_UP)
      RunRamp (Max,Min,Step,sleep_dur);
    else
      RunRamp (Min,Max,Step,sleep_dur);
  }
}


//---------------------------------------------------------
//---------------------------------------------------------
int WINAPI WinMain( HINSTANCE hInstance,
          HINSTANCE hPrevInstance,
          LPTSTR    lpCmdLine,
          int       nCmdShow)
{
  int i, numReps=50;
  
  RETAILMSG(1,(L"* Backlight_Test (ADS #%s)\r\n",VERSION));
  DEBUGMSG(1,(L"* - Debug build -\r\n"));

  if (int err=SelectLibrary())
  {
    RETAILMSG(1,(L"! Program aborted.\r\n\n"));
    exit(err);
  }

  RETAILMSG(1,(L"  Ramping Backlight PWM between max and min %d times.\r\n",numReps));
  RETAILMSG(1,(L"  Turns backlight off, then on when level is set to 255 (LS520 brightest).\r\n"));

  for (i=1;i<=numReps;i++)
  {
    RETAILMSG(1,(L"\r Cycle %d of %d...",i,numReps));
    DEBUGMSG(1,(L"\r\n"));  // next line for ramp debug messages

    // Ramp down, then back up again 
    // Use INVERSE b/c Xentek LS520 inverter is brightest at PWM=0

    BacklightRamp (RAMP_DOWN, INVERSE);

    BacklightRamp (RAMP_UP, INVERSE);

    DEBUGMSG(1,(L" Backlight off.\r\n"));
#if defined ON_OFF_LS520
    SIOBacklightOff();
#elif defined ON_OFF_LS520_REVERSED
    SIOBacklightOn();
#endif

    Sleep(500);

    DEBUGMSG(1,(L" Backlight on.\r\n"));
#if defined ON_OFF_LS520
    SIOBacklightOn();
#elif defined ON_OFF_LS520_REVERSED
    SIOBacklightOff();
#endif

  }

  RETAILMSG(1,(L"\r\nBacklight_Test: Complete\r\n"));

  return 0;
}

